# -*- coding: utf-8 -*-
from html import escape

from pytgbot.api_types.receivable.peer import Chat
from ....logger import logging


__author__ = 'luckydonald'

logger = logging.getLogger(__name__)
if __name__ == '__main__':
    logging.add_colored_handler(level=logging.DEBUG)
# end if


def format_chat(chat: Chat):
    """
    Formats a channel for html, escaping username and title.
    Will try to link to the ID if possible.

    :param chat:
    :return:
    """
    escaped_title = escape(chat.title) if chat.title is not None else None           # this is used about everywhere
    escaped_username = escape(chat.username) if chat.username is not None else None  # this is used about everywhere
    logger.debug(f'creating representation of {chat!r}')
    if 'link' in chat and chat.invite_link:
        channel_link = f"<a href={chat.invite_link!r}>{escaped_title}</a>"
    elif 'username' in chat and chat.username:
        channel_link = f"<a href=\"https://t.me/{escaped_username}\">{escaped_title}</a>"
    else:
        channel_link = f"<b>{escaped_title}</b>"
    # end if
    if 'id' in chat and chat.id:
        if 'username' in chat and chat.username:
            channel_link += f" (<code>{chat.id}</code>, @{escaped_username})"
        else:
            channel_link += f" (<code>{chat.id}</code>)"
        # end if
    else:  # id not in data
        if 'username' in chat and chat.username:
            channel_link += f" (@{escaped_username})"
        else:
            pass
            # channel_link += f" (<i>???</i>)"
        # end if
    # end if

    return channel_link
# end def



# -*- coding: utf-8 -*-
# tree.py
"""
 Written by Doug Dahms, modified by luckydonald 2016

 Prints the tree structure for the path specified on the command line

 http://code.activestate.com/recipes/217212-treepy-graphically-displays-the-directory-structur/
"""
# import os
from os import listdir, sep
from os.path import abspath, basename, isdir


def tree_linux(directory, padding="", print_files=False):
    """
    Not confirmed to work...
    """
    from os import popen
    return_string = ""
    cmd = "find '%s'" % dir
    files = popen(cmd).read().strip().split('\n')
    padding = '|  '
    for file in files:
        level = file.count(sep)
        pieces = file.split(sep)
        symbol = {0: '', 1: '/'}[isdir(file)]
        if not print_files and symbol != '/':
            continue
        return_string += padding * level + pieces[-1] + symbol
        print(padding * level + pieces[-1] + symbol)
        # end for
# end def


def tree(directory, padding="", print_files=False, level=-1, print_it=True, space=1,
         tree_letters=["║", "╟", "╙", "╴", "⚠"]):
    """
    Displays a tree of a folder the sub-folders. Optionally with files.
    :param directory: The path of the folder
    :param padding: What to put in front of the line
    :param print_files: weather it should include files, too.
    :param level: how many level it should print. -1 means unlimited.
    :param print_it: if it should print it, or just return the string.
    :param space: How many lines should be between files (in lines). Default: 1
    :param tree_letters: Pure ascii variant: "|++-". You can use a string instead of a list.
    :return:
    """
    from ..text import split_in_parts
    tree_letters = split_in_parts(tree_letters, 5)
    # print(tree_letters)
    return_string = ""
    if space < 0:
        space = 0
    # end if
    if level != -1:
        if level == 0:
            return return_string
        level -= 1
    # end if
    padding += ' '
    try:
        if print_files:
            files = listdir(directory)
        else:
            files = [x for x in listdir(directory) if isdir(directory + sep + x)]
            # end if
    except PermissionError:
        files = PermissionError
    # end try
    count = 0
    if files == PermissionError:
        string = (padding + tree_letters[0] + '\n') * space
        string += padding + tree_letters[2] + (tree_letters[3] if space == 0 else tree_letters[4]) + 'PERMISSION DENIED'
        if print_it:
            print(string)
        # end if
        return_string += string + "\n"
        files = []
    for file in files:
        count += 1
        string = padding + tree_letters[0]
        for i in range(space):
            if print_it:
                print(string)
            # end if
            return_string += string + "\n"
        # end for
        path = directory + sep + file
        if count == len(files):
            node = tree_letters[2] + tree_letters[3]
        else:
            node = tree_letters[1] + tree_letters[3]
        # end if
        if isdir(path):
            if count == len(files):  # last one
                return_string += echo(padding + tree_letters[2] + tree_letters[3] + basename(abspath(path)) + '/',
                                      print_files)
                return_string += tree(path, padding + "  ", print_files, level=level, space=space)
            else:
                return_string += echo(padding + tree_letters[1] + tree_letters[3] + basename(abspath(path)) + '/',
                                      print_files)
                return_string += tree(path, padding + tree_letters[0] + ' ', print_files, level=level, space=space)
                # end if
        else:
            string = padding + node + file
            return_string += echo(string, print_files)
    return return_string
# end def


def echo(msg, should_print):
    if should_print:
        print(msg)
    # end if
    return msg + '\n'
# end def


def usage():
    from sys import argv
    return '''Usage: {SCRIPTNAME} [-f] [<LEVELS>] <PATH>
Print tree structure of path specified.
Options:
-f      Print files as well as directories
LEVELS  How many levels to print. -1 means unlimited
PATH    Path to process'''.format(SCRIPTNAME=basename(argv[0]))
# end def


def main(*args):
    if not args:
        from sys import argv as args
    else:
        args = [__file__].extend(args)  # args[0] is the program
    if len(args) == 1:
        print(usage())
    elif len(args) == 2:
        # print just directories
        path = args[1]
        if isdir(path):
            tree(path, ' ')
        else:
            print('ERROR: \'' + path + '\' is not a directory')
    elif len(args) == 3:
        # 2 params
        if args[1] == '-f':
            # print directories and files
            path = args[2]
            if isdir(path):
                tree(path, ' ', True)
            else:
                print('ERROR: \'' + path + '\' is not a directory')
                # end if
        else:
            # print directories up to level
            try:
                levels = int(args[1])
            except Exception:
                print("ERROR: could not parse level '" + args[1] + " as integer")
            else:  # no Exception
                path = args[2]
                if isdir(path):
                    tree(path, ' ', True, level=levels)
                else:
                    print('ERROR: \'' + path + '\' is not a directory')
                    # end if
                    # end try
                    # end if
    elif len(args) == 4 and args[1] == '-f':
        # 3 params.
        # print directories and files up to level
        try:
            levels = int(args[2])
        except Exception:
            print("ERROR: could not parse level '" + args[1] + " as integer")
        else:  # no Exception
            path = args[3]
            if isdir(path):
                tree(path, ' ', True, level=levels)
            else:
                print('ERROR: \'' + path + '\' is not a directory')
                # end if
                # end try
    else:
        print(usage())


if __name__ == '__main__':
    main()



(
    CallerResult(
        self=CallerInfo(
            name='single_level',
            file='/Users/luckydonald/Documents/Programmieren/Python/luckydonald-utils.git/luckydonaldUtils/functions.py',
            line=82,
            code=None
        ),
        caller=CallerInfo(
            name='run',
            file='/Users/luckydonald/.pyenv/versions/3.7.2/lib/python3.7/unittest/case.py',
            line=615,
            code='                    testMethod()\n')),
    CallerResult(
        self=CallerInfo(
            name='single_level',
            file='/Users/luckydonald/Documents/Programmieren/Python/luckydonald-utils.git/tests/functions.py',
            line=7,
            code=None
        ), caller=CallerInfo(
            name='test_one_level',
            file='/Users/luckydonald/Documents/Programmieren/Python/luckydonald-utils.git/tests/functions.py',
            line=41,
            code='        result, result2 = single_level()\n'
        )
    )
)



# -*- coding: utf-8 -*-
""" Python 3 only logger part"""

__author__ = 'luckydonald'

def success(self, msg, *args, exc_info=False, **kwargs):
    """
    Log 'msg % args' with severity 'SUCCESS'.

    To pass exception information, use the keyword argument exc_info with
    a true value.

    logger.debug("Houston, we landed in the %s", "moon", exc_info=False)
    """
    kwargs["exc_info"] = exc_info
    self._success(msg, *args, **kwargs)


# -*- coding: utf-8 -*-
from html import escape
from typing import Union

from pytgbot.api_types.receivable.peer import User
from pytgbot.exceptions import TgApiException
from pytgbot import Bot

from ....logger import logging


__author__ = 'luckydonald'

logger = logging.getLogger(__name__)
if __name__ == '__main__':
    logging.add_colored_handler(level=logging.DEBUG)
# end if


def format_user(
    user: User, do_link: bool = False, prefer_username: bool = False, id_fallback: bool = False,
    user_tag: bool = None, id_tag: bool = None, html_escape: bool = True
) -> str:
    """
    Gets a user's name.

    If you want to use it in html you should probably keep html character escaping enabled (`html_escape=True`).
    You can specify to wrap the user name string or id string in html tags, with the user_tag or id_tag arguments.
    If the name is "Max Mustermann", with `user_tag="b"` it will return "<b>Max Mustermann</b>".

    Could return `None` if that user was either not found, or had no usable data.

    :param user: the telegram user object.
    :param do_link: If it should do a link instead of bold or anything provided in `user_tag` or `id_tag`.
    :param prefer_username: if you should prefer the @username over the real name
    :param id_fallback: if no real name and no @username exists, if it should fallback to the user id: user#123456
    :param user_tag: Name of the html tag to wrap the user string in (not the @username)
    :param id_tag: Name of the html tag to wrap the user id in. (`id_fallback` must be `True`)
    :param html_escape: If html tags in strings should be escaped. Default: True

    :return: the (formatted) name
    """
    assert isinstance(user, User)

    def maybe_escape(text):
        """
        None >-------------------------> None
        text >-- html_escape = True ---> escape(text)
        text >-- html_escape = False --> text
        """
        return escape(text) if (text is not None and html_escape) else text
    # end def

    name = None
    if user.first_name:
        if user.last_name:  # first + last
            name = maybe_escape(user.first_name + " " + user.last_name)
        else:  # only first
            name = maybe_escape(user.first_name)
        # end if
        if do_link:
            name = '<a href="tg://user?id={id}">{name}</a>'.format(id=user.id, name=name)
        elif user_tag:
            name = "<{tag}>{name}</{tag}>".format(tag=user_tag, name=name)
        # end if
    else:
        if user.last_name:  # only last
            name = maybe_escape(user.last_name)
            if do_link:
                name = '<a href="tg://user?id={id}">{name}</a>'.format(id=user.id, name=name)
            elif user_tag:
                name = "<{tag}>{name}</{tag}>".format(tag=user_tag, name=name)
            # end if
        # end if
    # end if

    # fallback if no fist/lastname or overwrite if prefer_username.
    if user.username and (name is None or prefer_username):
        name = "@" + maybe_escape(user.username)
    # end if

    # id fallback
    if not name and user.id and id_fallback:
        name = maybe_escape("user#" + str(user.id))
        if do_link:
            name = '<a href="tg://user?id={id}">{name}</a>'.format(id=user.id, name=name)
        elif id_tag:
            name = "<{tag}>{name}</{tag}>".format(tag=id_tag, name=name)
        # end if
    # end if
    return name
# end def


def retrieve_and_format_user(
    bot: Bot, user_id: Union[str, int], do_link: bool = False, prefer_username: bool = False, id_fallback: bool = False,
    user_tag: bool = None, id_tag: bool = None, html_escape: bool = True
) -> str:
    """
    Gets a user's name by given user id.

    If you want to use it in html you should probably keep html character escaping enabled (`html_escape=True`).
    You can specify to wrap the user name string or id string in html tags, with the user_tag or id_tag arguments.
    If the name is "Max Mustermann", with `user_tag="b"` it will return "<b>Max Mustermann</b>".

    Could return `None` if that user was either not found, or had no usable data.

    :param bot: The bot instance to use to check the user.
    :param user_id: the id of the telegram user.
    :param do_link: If it should do a link instead of bold or anything provided in `user_tag` or `id_tag`.
    :param prefer_username: if you should prefer the @username over the real name
    :param id_fallback: if no real name and no @username exists, if it should fallback to the user id: user#123456
    :param user_tag: Name of the html tag to wrap the user string in (not the @username)
    :param id_tag: Name of the html tag to wrap the user id in. (`id_fallback` must be `True`)
    :param html_escape: If html tags in strings should be escaped. Default: True

    :return: the (formatted) name
    """
    try:
        user = bot.get_chat_member(user_id, user_id).user
    except TgApiException:
        if id_fallback:
            name = "user#" + str(user_id)
            name = escape(name) if (name is not None and html_escape) else name
            if do_link:
                return '<a href="tg://user?id={id}">{name}</a>'.format(id=user_id, name=name)
            if id_tag:
                return "<{tag}>{name}</{tag}>".format(tag=id_tag, name=name)
            # end if
        # end if
        return None
    # end try
    return format_user(
        user,
        do_link=do_link, prefer_username=prefer_username, id_fallback=id_fallback,
        user_tag=user_tag, id_tag=id_tag, html_escape=html_escape
    )
# end def



def create_hierarchic_dict(files, separator="/"):
    tree = {}
    for file in files:
        if file is None:
            continue
        # end if
        tree_pointer = tree
        parts = file.split(separator)
        for part in parts:
            if part not in tree_pointer:
                tree_pointer[part] = {}
            # end if
            tree_pointer = tree_pointer[part]
        # end for
    # end for
    return tree


# end def


def hierarchic_dict_simplify_paths(tree_pointer, separator="/"):
    for cur_key in list(tree_pointer.keys()):
        hierarchic_dict_simplify_paths(tree_pointer[cur_key], separator=separator)
        if len(tree_pointer[cur_key]) == 1:  # only one node, can be flattened.
            sub_key = list(tree_pointer[cur_key].keys())[0]
            new_key = cur_key + separator + sub_key
            tree_pointer[new_key] = tree_pointer.pop(cur_key)[sub_key]
        # end if
    # end for
# end def



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import sys

__author__ = 'luckydonald'

if sys.version < '3':  # python 2.7
    text_type = unicode
    binary_type = str
    native_type = binary_type
    unicode_type = text_type  # because I can't remember to use text_type
    long_int = long

    def to_native(x):
        return to_binary(x)
        # end def
else:  # python 3
    text_type = str
    binary_type = bytes
    native_type = text_type
    unicode_type = text_type
    long_int = int

    def to_native(x):
        return to_unicode(x)
        # end def
# end if


def to_binary(x):
    if isinstance(x, text_type):
        return x.encode("utf-8")
    elif isinstance(x, binary_type):
        return x
    else:
        return to_binary(str(x))  # str() can fail.  # do i need bytes() here with py3 (because ascii and stuff?) ?
        # end if


# end def


def to_unicode(x):
    if isinstance(x, binary_type):
        return x.decode("utf-8")
    elif isinstance(x, text_type):
        return x
    else:
        return to_unicode(str(x))  # str() can fail.
        # end if

# end def



# -*- coding: utf-8 -*-
import re

__author__ = 'luckydonald'

from ..github import _USER_NAME_REGEX, _REPO_NAME_REGEX

# https://regex101.com/r/gH8xM9/4
_SIMPLE_URL_REGEX = r"(?:https?://(?:www\.)?|www\.|^|(?<=\s))github.com/[^\s]+"
SIMPLE_URL_REGEX = re.compile(_SIMPLE_URL_REGEX)
""" :const:`SIMPLE_URL_REGEX`: A very simple url regex """

# https://regex101.com/r/qK6uN9/15
_FILE_URL_REGEX = (
        r"\b(?P<url>(?P<protocol>https?://)?(?P<www>www\.)?github.com/"
        r"(?P<user>" + _USER_NAME_REGEX + ")/"
                                          r"(?P<repo>" + _REPO_NAME_REGEX + ")/?"
                                                                            r"(?P<path>(?:(?P<kind>blob|tree)/(?P<branch>[a-zA-Z0-9_%-]+))/?(?P<file>[^\s#]*)/?)?)"
                                                                            r"(?:#(?P<hash>[-;/?:@&=+$,_.!~*'()%0-9a-zA-Z]*))?(?:(?=\s)|$)"
)
FILE_URL_REGEX = re.compile(  # https://regex101.com/r/qK6uN9/15
    _FILE_URL_REGEX, flags=re.IGNORECASE
)
"""
:const:`FILE_URL_REGEX`: Matching github urls with the following named matching groups:

- `url`: the complete url

    - `protocol`: "https://" or "http://" or empty/non-existent
    - `user`: git user or organisation
    - `repo`: the repository
    - `path`: When existent, this is not the project page (root of master)

        - `kind`: blob or tree
        - `branch`: the name of the branch (kind=tree), or the commit hash (kind=blob)
        - `file`: the rest of the filepath (from root of that branch, can be empty)

    - `hash`: Can be non-existent or empty. Everything behind the '#'
"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import json
import requests

from luckydonaldUtils.logger import logging

from pytgbot.api_types.receivable.media import Sticker
from pytgbot.api_types.receivable.updates import Message, Update

from teleflask import TBlueprint

from ...regex.urls.telegram import ADDSTICKERS_REGEX

__author__ = 'luckydonald'


logger = logging.getLogger(__name__)
if __name__ == '__main__':
    logging.add_colored_handler(level=logging.DEBUG)
# end if

GETSTICKERS_API_KEY = os.getenv('GETSTICKERS_API_KEY', None)
GETSTICKERS_DOMAIN = os.getenv('GETSTICKERS_DOMAIN', 'https://getstickers.me')   # so we can switch to staging there.
GETSTICKERS_ENABLED = GETSTICKERS_API_KEY and GETSTICKERS_DOMAIN

TIMEOUT = 0.5

sticker_crawl_tbp = TBlueprint(__name__)

if not GETSTICKERS_ENABLED:
    logger.warning('Did not register sticker crawler as GETSTICKERS_API_KEY is not defined.')
    raise ImportError('Did not register sticker crawler as GETSTICKERS_API_KEY is not defined.')
# end if


@sticker_crawl_tbp.on_message
def on_sticker_message(update: Update, msg: Message):
    submit_sticker_message(msg)
    collect_pack_text(msg)
    return
# end def


@sticker_crawl_tbp.on_update('channel_post')
def on_sticker_update_channel_post(update: Update):
    submit_sticker_message(update.channel_post)
    collect_pack_text(update.channel_post)
    return
# end def


@sticker_crawl_tbp.on_update('edited_message')
def on_sticker_update_edited_message(update: Update):
    submit_sticker_message(update.edited_message)
    collect_pack_text(update.edited_message)
    return
# end def


@sticker_crawl_tbp.on_update('edited_channel_post')
def on_sticker_update_edited_channel_post(update: Update):
    submit_sticker_message(update.edited_channel_post)
    collect_pack_text(update.edited_channel_post)
    return
# end def


def collect_pack_text(message: Message):
    if not message.text or '/addstickers/' not in message.text:
        return
    # end if
    for match in ADDSTICKERS_REGEX.finditer(message.text):
        submit_pack(match.group('pack'))
    # end for
# end def


def submit_pack(pack_url: str):
    try:
        requests.put(
            GETSTICKERS_DOMAIN + '/api/v3/submit/pack/' + pack_url,
            params={
                "key": GETSTICKERS_API_KEY,
                "bot_id": sticker_crawl_tbp.user_id,
                "pack": pack_url,
            },
            timeout=TIMEOUT,
        )
    except requests.HTTPError as e:
        try:
            result = repr(e.response.json())
        except:
            result = e.response.text
        # end try
        logger.warning(f'Submitting sticker to getstickers.me failed with error code {e.response.status_code}: {result}')
    except:
        logger.warning('Submitting sticker to getstickers.me failed.', exc_info=True)
    # end try
# end def


def submit_sticker_message(message: Message):
    if not isinstance(message, Message) or not isinstance(message.sticker, Sticker):
        return
    # end if
    payload = json.dumps(message.to_array())
    logger.debug(f'sending {payload!r} to the API.')
    try:
        requests.put(
            GETSTICKERS_DOMAIN + '/api/v3/submit/sticker',
            params={
                "key": GETSTICKERS_API_KEY,
                "bot_id": sticker_crawl_tbp.user_id,
            },
            data={
                "message": payload,
            },
            timeout=TIMEOUT,
        )
        return
    except requests.HTTPError as e:
        try:
            result = repr(e.response.json())
        except:
            result = e.response.text
        # end try
        logger.warning(f'Submitting sticker to getstickers.me failed with error code {e.response.status_code}: {result}')
    except:
        logger.warning('Submitting sticker to getstickers.me failed.', exc_info=True)
    # end try

    # so it failed.
    if message.sticker.set_name:
        submit_pack(message.sticker.set_name)
    # end if
# end def




# -*- coding: utf-8 -*-

from ...eastereggs.headers import get_headers as get_easteregg_headers
from ...logger import logging  # pip install luckydonald-utils
__author__ = 'luckydonald'

logger = logging.getLogger(__name__)

class EastereggHeadersMiddleware(object):
    """
    Sets some funny headers.

    Include in your `MIDDLEWARE_CLASSES: "luckydonaldUtils.djangos.middleware.headers.EastereggHeadersMiddleware"
    """
    def process_response(self, request, response):
        for header_key, header_value in get_easteregg_headers().items():
            try:
                if not isinstance(header_value, str):
                    raise TypeError("Keyword-parameter '{param}' is not type str but {type}, and str() failed.".format(param=header_key, type=type(header_value)))
                if isinstance(header_value, str):
                    response[header_key.replace("_","-")] = header_value
                else:
                    raise TypeError("Keyword-parameter '{param}' is not type str but {type}.".format(param=header_key, type=type(header_value)))
            except TypeError:
                logger.exception("Header error")
                continue
            # end try
        # end for
        return response
    # end def
# end class EastereggHeadersMiddleware



# -*- coding: utf-8 -*-

import base64
from .dependencies import import_or_install

try:
    from usersettings import Settings
except ImportError:  # pragma nocover
    Settings = import_or_install("usersettings.Settings", "usersettings")  # pip install usersettings
# end try

try:
    from Crypto import Random
    from Crypto.Cipher import AES
    from Crypto.Hash import MD5
except ImportError:  # pragma nocover
    Random = import_or_install("Crypto.Random", "pycrypto")  # pip install pycrypto
    AES = import_or_install("Crypto.Cipher.AES", "pycrypto")  # pip install pycrypto
    MD5 = import_or_install("Crypto.Hash.MD5", "pycrypto")  # pip install pycrypto
# end try

__author__ = 'luckydonald'

BS = 16


def pad(s):
    return s + (BS - len(s) % BS) * chr(BS - len(s) % BS)


# end def

def un_pad(s):
    return s[:-ord(s[len(s) - 1:])]


# end def


class Store(object):
    def __init__(self, settings_name, key=None):
        if not key:
            settings = Settings(settings_name)  # store settings, password etc.
            settings.add_setting("do-not-change!", str, random())
            settings.load_settings()
            settings.save_settings()
            self.key = settings.get("do-not-change!")
        else:
            self.key = key

    def encrypt(self, raw):
        raw = pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return base64.b64encode(iv + cipher.encrypt(raw))

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        iv = enc[:16]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return un_pad(cipher.decrypt(enc[16:]))


def random():
    return MD5.new(Random.new().read(4)).hexdigest()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from luckydonaldUtils.logger import logging

__author__ = 'luckydonald'

logger = logging.getLogger(__name__)
if __name__ == '__main__':
    logging.add_colored_handler(level=logging.DEBUG)
# end if




# -*- coding: utf-8 -*-
from typing import Union

from ....dependencies import import_or_install

try:
    from pytgbot import Bot
    from pytgbot.api_types.receivable.peer import ChatMember
    from pytgbot.exceptions import TgApiException
except ImportError:  # pragma nocover
    Bot = import_or_install("pytgbot.bot", "Bot")  # pip install pytgbot
    ChatMember = import_or_install("pytgbot.api_types.receivable.peer", "ChatMember")  # pip install pytgbot
    TgApiException = import_or_install("pytgbot.exceptions", "TgApiException")  # pip install pytgbot
# end try


from luckydonaldUtils.logger import logging

__author__ = 'luckydonald'

logger = logging.getLogger(__name__)
if __name__ == '__main__':
    logging.add_colored_handler(level=logging.DEBUG)
# end if


STATUS_IS_ADMIN = ("creator", "administrator")


def is_admin(chat_member: ChatMember, right: Union[None, str] = None) -> bool:
    """
    Checks if a user is admin (or even creator) of a chat/group. Optionally a specific right can be requested to exist.

    :param chat_member: The ChatMember object to decide upon.
    :param right: Right to require, e.g. `"can_promote_members"`.

    :return:
    """
    if chat_member.status not in STATUS_IS_ADMIN:
        return False  # needs to be admin
    # end if
    if right:  # so we wanna know if he has the right permissions.
        assert hasattr(chat_member, right)  # so the field exists.
        logger.debug('getattr(chat_member, right)' + repr(getattr(chat_member, right)) + ": " + repr(getattr(chat_member, right) == True))
        return getattr(chat_member, right) is True  # get value
    # end if
    return True
# end def


def retrieve_and_is_admin(bot: Bot, chat_id: int, user_id: int, right: Union[None, str] = None) -> bool:
    """
    Checks if a user is admin (or even creator) of a chat/group. Optionally a specific right can be requested to exist.
    If lookup fails, `False` is assumed.

    :param bot: The bot to execute the get_chat_member request with.
    :param chat_id:
    :param user_id:
    :param right: Right to require, e.g. `"can_promote_members"`.

    :return:
    """
    try:
        chat_member = bot.get_chat_member(chat_id, user_id)
        logger.debug(f'get_chat_member result: {chat_member}')
        return is_admin(chat_member, right=right)
    except TgApiException:
        logger.exception('calling get_chat_member failed. Defaulting to \'no admin\'.')
        return False
    # end try
# end def



# -*- coding: utf-8 -*-

from __future__ import division

__author__ = 'luckydonald'

from .dependencies import import_or_install
from .files import gettempdir
from .encoding import to_binary as b
import hashlib
import mimetypes
import sys
import os

from DictObject import DictObject

try:
    import magic
except ImportError:  # pragma nocover
    magic = import_or_install("magic", "python-magic")  # import magic
# end try

try:
    import requests
    from requests.packages.urllib3.exceptions import HTTPError
except ImportError:  # pragma nocover
    requests = import_or_install("requests", "requests")  # import requests
    HTTPError = import_or_install("requests.packages.urllib3.exceptions.HTTPError", "requests")
# end try

try:
    import progressbar
    from progressbar import Widget
    from progressbar import ETA, Percentage, Bar, FileTransferSpeed, ProgressBar
except ImportError:  # pragma nocover
    progressbar_version = ("progressbar" if sys.version < "3.3" else "progressbar33")
    import_or_install("progressbar", progressbar_version)
    Widget = import_or_install("progressbar.Widget import Widget", progressbar_version)
    ETA = import_or_install("progressbar import ETA", progressbar_version)
    Percentage = import_or_install("progressbar import Percentage", progressbar_version)
    Bar = import_or_install("progressbar import Bar", progressbar_version)
    FileTransferSpeed = import_or_install("progressbar import FileTransferSpeed", progressbar_version)
    ProgressBar = import_or_install("progressbar import ProgressBar", progressbar_version)
# end try

import logging

logger = logging.getLogger(__name__)

HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X10.69; rv:4458.42) Gecko/4458 Firefox/69.0 Pon3Downloader'}


def get_json(url, objectify=True, **kwargs):
    kwargs.setdefault("headers", HEADERS)
    json = requests.get(url, **kwargs).json()
    if objectify:
        return DictObject.objectify(json)
    return json


def download_file(url, used_cached=True, download_folder=None, temp_folder_name="luckydonald-utils", return_mime=False,
                  return_buffer=False, progress_bar=False, **requests_kwargs):
    """
    Downloads a file. Filename is the md5 hash of the url.

    :param url: The url you want to download.

    :param used_cached: If the url was already downloaded, that file will not be overwritten.
    The files name is just a hash of the url. So the same filename means same hash means same url.

    :param download_folder:
    Path to the download dir.
    If None, a folder will be created inside the /tmp/ folder equivalent of your system, with the name from `temp_folder_name`.

    :param temp_folder_name:
    Name of the folder to be created/used in the /tmp/ equivalent of your system.
    This is option ignored, if `download_folder` is given. 	Default: "luckydonald-utils"

    :param return_mime: If set to true, a (filename, mime) tuple is returned, else just the filename.

    :param return_buffer: If set to true, nothing will be written to disk,
    you'll get the buffer returned instead of the filename.

    :param progress_bar: If it should display a progress bar.

    :param **requests_kwargs: All other arguments are piped to the requests.get() call.

    :return: The path of the file. If `return_buffer` is true, instead it contains the buffer.
    If `return_mime` is true, it is a tuple of that and the mime, (file_path, mime) or (buffer, mime)

    """
    requests_kwargs.setdefault("headers", HEADERS)
    bar = Bar(marker="#", left="[", right="]")
    if not return_buffer:
        if not download_folder:
            download_folder = gettempdir(temp_folder_name=temp_folder_name)
        # end if not return_buffer
    try:
        logger.debug("DL: Downloading from '{url}'.".format(url=url))
        if progress_bar:
            requests_kwargs["stream"] = True
            response = requests.get(url, **requests_kwargs)
            total_length = response.headers.get('content-length')
            if total_length is None:  # no content length header
                widgets = ["Downloading Song: ", bar, " ",
                           DoneWidget(init="Connecting...", started="Unknown Size.", finished="Complete.")]
                pbar = ProgressBar(widgets=widgets).start()
                logger.debug("No content-length provided.")
                image_buffer = response.content
                pbar.finish()
            else:
                dl = 0
                image_buffer = b("")
                total_length = int(total_length)
                widgets = ["Downloading Song: ", bar, " ", Percentage(), " ", FileTransferSpeed(), ", ", ETA()]
                pbar = ProgressBar(widgets=widgets, maxval=total_length).start()
                for data in response.iter_content(chunk_size=int(total_length / 100)):
                    dl += len(data)
                    image_buffer += data
                    pbar.update(dl)
                # end for
                pbar.finish()
                # now is downloaded.
                logger.debug("Download completed.")
        else:
            image_buffer = requests.get(url, **requests_kwargs).content
    except HTTPError as e:
        logger.exception("DL: Error in URL '" + url + "'.\n" + str(e))
        raise
        # return (None, None) if return_mime else None
    except Exception as e:
        logger.exception("DL: Error in Download '" + url + "'.\n" + str(e))
        raise
        # return (None, None) if return_mime else None
    mime = magic.from_buffer(image_buffer, mime=True).decode("utf-8")
    suffix = (mimetypes.guess_extension(mime) if mime else ".unknown") or ".unknown"
    suffix = '.jpg' if suffix == '.jpe' else suffix  # nobody uses that.
    if return_buffer:
        logger.debug("DL: Requested Buffer, not creating/checking dirs/files.")
        if return_mime:
            return image_buffer, mime
        return image_buffer
    else:  # -> not return_buffer:
        file_name = str(hashlib.md5(url.encode()).hexdigest()) + suffix
        file_name = os.path.join(download_folder, file_name)
        if os.path.isfile(file_name):
            if used_cached:
                logger.debug("DL: File exists, using cached: %s" % file_name)
                if return_mime:
                    return file_name, mime
                return file_name
            logger.debug("DL: File exists, redownloading: %s" % file_name)
        else:
            logger.debug("DL: File does not exist, downloading: %s" % file_name)
            if not os.path.exists(os.path.dirname(file_name)):
                logger.debug("DL: Download Folder does not exists. Creating.")
                os.makedirs(os.path.dirname(file_name))
        logger.debug("DL: Writing download from '%s' to file '%s'" % (url, file_name))
        try:
            with open(file_name, 'wb') as f:
                f.write(image_buffer)
        except Exception as e:
            logger.error("DL: Error in writing download to disk: '" + url + "' to '" + file_name + "'.\n" + str(e))
            file_name = None
        if return_mime:
            return file_name, mime
        return file_name
        # end if-else


class DoneWidget(Widget):
    """
    Can display 3 values.
    """

    def __init__(self, init="Not started.", started="Running.", finished="Done."):
        self.init_text = init
        self.started_text = started
        self.finished_text = finished

    def update(self, pbar):
        if pbar.start_time is None:
            return self.init_text
        elif not pbar.finished:
            return self.started_text
        else:
            return self.finished_text



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import sys

__author__ = 'luckydonald'
__all__ = ['py2', 'py3']

if sys.version < '3':  # python 2.7
    py2 = True
    py3 = False
else:
    py2 = False
    py3 = True  # This won't work with a future python 4, though.
# end if



# -*- coding: utf-8 -*-
from typing import Union

from ....dependencies import import_or_install

try:
    from pytgbot import Bot
    from pytgbot.api_types.receivable.peer import ChatMember
    from pytgbot.exceptions import TgApiException
except ImportError:  # pragma nocover
    Bot = import_or_install("pytgbot.bot", "Bot")  # pip install pytgbot
    ChatMember = import_or_install("pytgbot.api_types.receivable.peer", "ChatMember")  # pip install pytgbot
    TgApiException = import_or_install("pytgbot.exceptions", "TgApiException")  # pip install pytgbot
# end try


from luckydonaldUtils.logger import logging

__author__ = 'luckydonald'

logger = logging.getLogger(__name__)
if __name__ == '__main__':
    logging.add_colored_handler(level=logging.DEBUG)
# end if


STATUS_IS_GONE = ("left", "kicked")


def is_member(chat_member: ChatMember) -> bool:
    """
    Checks if a user is still in a chat.

    :param chat_member: The ChatMember object to decide upon.

    :return:
    """
    if chat_member.status in STATUS_IS_GONE:
        return False
    # end if
    return True
# end def


def retrieve_and_is_member(bot: Bot, chat_id: int, user_id: int) -> bool:
    """
    Checks if a user is still in a chat/group.
    If lookup fails, `False` is assumed.

    :param bot: The bot to execute the get_chat_member request with.
    :param chat_id: id of the chat the user should be in
    :param user_id: id of the user

    :return:
    """
    try:
        chat_member = bot.get_chat_member(chat_id, user_id)
        logger.debug(f'get_chat_member result: {chat_member}')
        return is_member(chat_member)
    except TgApiException:
        logger.exception('calling get_chat_member failed. Defaulting to \'no admin\'.')
        return False
    # end try
# end def
